@@ -2,6 +2,7 @@ from __future__ import annotations
 from uuid import uuid4
 import contextlib
 import shutil
+from pathlib import Path
 import json
 import logging
 import os
@@ -9,12 +10,11 @@ import random
 import re
 import time
 from concurrent.futures import Future, ThreadPoolExecutor, as_completed
-from pathlib import Path
 from typing import Any, Callable, Dict, Generator, List, Optional, Set, Tuple
+from github.CheckRun import CheckRun
 from stringcolor import cs
 
 from github.Branch import Branch
-from github.ContentFile import ContentFile
 import requests
 import shortuuid
 from starlette.datastructures import Headers
@@ -192,7 +192,6 @@ Available user actions:
             return
 
         event_log: str = f"Event type: {self.github_event}. event ID: {self.x_github_delivery}"
-        self.owners_content = self.get_owners_content()
 
         try:
             self.pull_request = self._get_pull_request()
@@ -201,7 +200,11 @@ Available user actions:
             self.last_commit = self._get_last_commit()
             self.parent_committer = self.pull_request.user.login
             self.last_committer = getattr(self.last_commit.committer, "login", self.parent_committer)
+            self.changed_files = self.list_changed_commit_files()
             self.pull_request_branch = self.pull_request.base.ref
+            self.all_approvers_and_reviewers = self.get_all_approvers_and_reviewers()
+            self.all_approvers = self.get_all_approvers()
+            self.all_reviewers = self.get_all_reviewers()
 
             if self.jira_enabled_repository:
                 self.set_jira_in_pull_request()
@@ -302,20 +305,22 @@ Available user actions:
 
     def prepare_log_prefix(self, pull_request: Optional[PullRequest] = None) -> str:
         _repository_color = self._get_reposiroty_color_for_log_prefix()
-        _id = self.x_github_delivery.split("-", 1)[-1]
         return (
-            f"{_repository_color}[{self.github_event}][{_id}][PR {pull_request.number}]:"
+            f"{_repository_color}[{self.github_event}][{self.x_github_delivery}][PR {pull_request.number}]:"
             if pull_request
-            else f"{_repository_color}[{self.github_event}][{_id}]:"
+            else f"{_repository_color}[{self.github_event}][{self.x_github_delivery}]:"
         )
 
     def process_pull_request_check_run_webhook_data(self) -> None:
         _check_run: Dict[str, Any] = self.hook_data["check_run"]
-        if _check_run["action"] != "completed":
-            self.logger.debug(f"{self.log_prefix} check run action is not completed, skipping")
+        check_run_name: str = _check_run["name"]
+
+        if self.hook_data.get("action", "") != "completed":
+            self.logger.debug(
+                f"{self.log_prefix} check run {check_run_name} action is {self.hook_data.get('action', 'N/A')} and not completed, skipping"
+            )
             return
 
-        check_run_name: str = _check_run["name"]
         check_run_status: str = _check_run["status"]
         check_run_conclusion: str = _check_run["conclusion"]
         check_run_head_sha: str = _check_run["head_sha"]
@@ -531,7 +536,7 @@ Available user actions:
             self.logger.error(f"{self.log_prefix} {err} - {_err}, {_out}")
             self.repository.create_issue(
                 title=_err,
-                assignee=self.approvers[0] if self.approvers else "",
+                assignee=self.root_approvers[0] if self.root_approvers else "",
                 body=f"""
 stdout: `{_out}`
 stderr: `{_err}`
@@ -575,69 +580,27 @@ stderr: `{_err}`
 """
                 self.send_slack_message(message=message, webhook_url=self.slack_webhook_url)
 
-    def get_owners_content(self) -> Dict[str, Any]:
-        try:
-            owners_content: list[ContentFile] | ContentFile = self.repository.get_contents("OWNERS")
-            if isinstance(owners_content, list):
-                self.logger.debug(f"{self.log_prefix} Found more then one OWNERS file, using the first one")
-                owners_content = owners_content[0]
-
-            _content: Dict[str, Any] = yaml.safe_load(owners_content.decoded_content)
-            self.logger.debug(f"{self.log_prefix} OWNERS file content: {_content}")
-            return _content
-
-        except UnknownObjectException:
-            self.logger.error(f"{self.log_prefix} OWNERS file not found")
-            return {}
-
     @property
-    def reviewers(self) -> List[str]:
-        bc_reviewers: List[str] = self.owners_content.get("reviewers", [])
-        if isinstance(bc_reviewers, dict):
-            _reviewers: List[str] = self.owners_content.get("reviewers", {}).get("any", [])
-        else:
-            _reviewers = bc_reviewers
-
-        self.logger.debug(f"{self.log_prefix} Reviewers: {_reviewers}")
+    def root_reviewers(self) -> List[str]:
+        _reviewers = self.all_approvers_and_reviewers.get(".", {}).get("reviewers", [])
+        self.logger.debug(f"{self.log_prefix} ROOT Reviewers: {_reviewers}")
         return _reviewers
 
     @property
-    def files_reviewers(self) -> Dict[str, str]:
-        _reviewers = self.owners_content.get("reviewers", {})
-        if isinstance(_reviewers, dict):
-            return _reviewers.get("files", {})
-
-        return {}
-
-    @property
-    def folders_reviewers(self) -> Dict[str, str]:
-        _reviewers = self.owners_content.get("reviewers", {})
-        if isinstance(_reviewers, dict):
-            return _reviewers.get("folders", {})
-
-        return {}
-
-    @property
-    def approvers(self) -> List[str]:
-        return self.owners_content.get("approvers", [])
+    def root_approvers(self) -> List[str]:
+        _approvers = self.all_approvers_and_reviewers.get(".", {}).get("approvers", [])
+        self.logger.debug(f"{self.log_prefix} ROOT Approvers: {_approvers}")
+        return _approvers
 
     def list_changed_commit_files(self) -> list[str]:
         return [fd["filename"] for fd in self.last_commit.raw_data["files"]]
 
     def assign_reviewers(self) -> None:
         self.logger.info(f"{self.log_prefix} Assign reviewers")
-        changed_files = self.list_changed_commit_files()
-        reviewers_to_add = self.reviewers
-        for _file, _file_reviewers in self.files_reviewers.items():
-            if _file in changed_files:
-                reviewers_to_add.extend(_file_reviewers)
 
-        for _folder, _folder_reviewers in self.folders_reviewers.items():
-            if any(cf for cf in changed_files if _folder in str(Path(cf).parent)):
-                reviewers_to_add.extend(_folder_reviewers)
-
-        _to_add: List[str] = list(set(reviewers_to_add))
+        _to_add: List[str] = list(set(self.all_reviewers))
         self.logger.debug(f"{self.log_prefix} Reviewers to add: {', '.join(_to_add)}")
+
         for reviewer in _to_add:
             if reviewer != self.pull_request.user.login:
                 self.logger.debug(f"{self.log_prefix} Adding reviewer {reviewer}")
@@ -856,7 +819,7 @@ stderr: `{_err}`
             self.logger.error(f"{self.log_prefix} Failed to delete tag: {repository_full_tag}. OUT:{out}. ERR:{err}")
 
     def process_comment_webhook_data(self) -> None:
-        if comment_action := self.hook_data["action"] in ("action", "deleted"):
+        if comment_action := self.hook_data["action"] in ("edited", "deleted"):
             self.logger.debug(f"{self.log_prefix} Not processing comment. action is {comment_action}")
             return
 
@@ -903,10 +866,12 @@ stderr: `{_err}`
                 pull_request_opened_futures.append(executor.submit(self.process_opened_or_synchronize_pull_request))
                 if self.jira_track_pr:
                     pull_request_opened_futures.append(executor.submit(self.create_jira_when_open_pull_reques))
+
                 pull_request_opened_futures.append(executor.submit(self.set_pull_request_automerge))
 
-            for _ in as_completed(pull_request_opened_futures):
-                pass
+            for result in as_completed(pull_request_opened_futures):
+                if _exp := result.exception():
+                    self.logger.error(f"{self.log_prefix} {_exp}")
 
         if hook_action == "synchronize":
             pull_request_synchronize_futures: List[Future] = []
@@ -919,6 +884,10 @@ stderr: `{_err}`
                 if self.jira_track_pr:
                     pull_request_synchronize_futures.append(executor.submit(self.update_jira_when_pull_request_sync))
 
+            for result in as_completed(pull_request_synchronize_futures):
+                if _exp := result.exception():
+                    self.logger.error(f"{self.log_prefix} {_exp}")
+
         if hook_action == "closed":
             self.close_issue_for_merged_or_closed_pr(hook_action=hook_action)
             self.delete_remote_tag_for_merged_or_closed_pr()
@@ -960,7 +929,7 @@ stderr: `{_err}`
                 _reviewer = labeled.split(CHANGED_REQUESTED_BY_LABEL_PREFIX)[-1]
 
             _approved_output: Dict[str, Any] = {"title": "Approved", "summary": "", "text": ""}
-            if _reviewer in self.approvers:
+            if _reviewer in self.all_approvers:
                 _check_for_merge = True
                 _approved_output["text"] += f"Approved by {_reviewer}.\n"
 
@@ -1016,7 +985,7 @@ stderr: `{_err}`
         label_prefix: str = ""
         label_to_remove: str = ""
 
-        if reviewed_user in self.approvers:
+        if reviewed_user in self.all_approvers:
             approved_lgtm_label = APPROVED_BY_LABEL_PREFIX
         else:
             approved_lgtm_label = LGTM_BY_LABEL_PREFIX
@@ -1175,7 +1144,7 @@ stderr: `{_err}`
 
         elif _command == HOLD_LABEL_STR:
             self.create_comment_reaction(issue_comment_id=issue_comment_id, reaction=REACTIONS.ok)
-            if reviewed_user not in self.approvers:
+            if reviewed_user not in self.all_approvers:
                 self.pull_request.create_issue_comment(
                     f"{reviewed_user} is not part of the approver, only approvers can mark pull request as hold"
                 )
@@ -1281,13 +1250,9 @@ stderr: `{_err}`
         for pull_request in self.repository.get_pulls(state="open"):
             self.pull_request = pull_request
             self.logger.info(f"{self.log_prefix} check label pull request after merge")
-            self.label_pull_request_by_merge_state(_sleep=time_sleep)
-
-    def label_pull_request_by_merge_state(self, _sleep: int = 0) -> None:
-        if _sleep:
-            self.logger.info(f"{self.log_prefix} Sleep for {_sleep} seconds before checking merge state")
-            time.sleep(_sleep)
+            self.label_pull_request_by_merge_state()
 
+    def label_pull_request_by_merge_state(self) -> None:
         merge_state = self.pull_request.mergeable_state
         self.logger.debug(f"{self.log_prefix} Mergeable state is {merge_state}")
         if merge_state == "unknown":
@@ -1327,85 +1292,39 @@ stderr: `{_err}`
         failure_output = ""
 
         try:
-            self.all_required_status_checks = self.get_all_required_status_checks()
             self.logger.info(f"{self.log_prefix} Check if {CAN_BE_MERGED_STR}.")
             self.set_merge_check_queued()
             last_commit_check_runs = list(self.last_commit.get_check_runs())
-            self.logger.debug(f"{self.log_prefix} Check if any required check runs in progress.")
-            check_runs_in_progress = [
-                check_run.name
-                for check_run in last_commit_check_runs
-                if check_run.status == IN_PROGRESS_STR
-                and check_run.name != CAN_BE_MERGED_STR
-                and check_run.name in self.all_required_status_checks
-            ]
-            if check_runs_in_progress:
-                self.logger.debug(
-                    f"{self.log_prefix} Some required check runs in progress {check_runs_in_progress}, "
-                    f"skipping check if {CAN_BE_MERGED_STR}."
-                )
-                failure_output += f"Some required check runs in progress {', '.join(check_runs_in_progress)}\n"
-
             _labels = self.pull_request_labels_names()
-            is_hold = HOLD_LABEL_STR in _labels
-            is_wip = WIP_STR in _labels
-            if is_hold or is_wip:
-                if is_hold:
-                    failure_output += "Hold label exists.\n"
-
-                if is_wip:
-                    failure_output += "WIP label exists.\n"
-
-            if not self.pull_request.mergeable:
-                failure_output += "PR is not mergeable: {self.pull_request.mergeable_state}\n"
-
-            failed_check_runs = []
-            for check_run in last_commit_check_runs:
-                if (
-                    check_run.name == CAN_BE_MERGED_STR
-                    or check_run.conclusion == SUCCESS_STR
-                    or check_run.conclusion == QUEUED_STR
-                    or check_run.name not in self.all_required_status_checks
-                ):
-                    continue
-
-                failed_check_runs.append(check_run.name)
-
-            if failed_check_runs:
-                exclude_in_progress = [
-                    failed_check_run
-                    for failed_check_run in failed_check_runs
-                    if failed_check_run not in check_runs_in_progress
-                ]
-                failure_output += f"Some check runs failed: {', '.join(exclude_in_progress)}\n"
-
             self.logger.debug(f"{self.log_prefix} check if can be merged. PR labels are: {_labels}")
 
-            for _label in _labels:
-                if CHANGED_REQUESTED_BY_LABEL_PREFIX.lower() in _label.lower():
-                    change_request_user = _label.split("-")[-1]
-                    if change_request_user in self.approvers:
-                        failure_output += "PR has changed requests from approvers\n"
+            is_pr_mergable = self.pull_request.mergeable
+            if not is_pr_mergable:
+                failure_output += f"PR is not mergeable: {is_pr_mergable}\n"
 
-            missing_required_labels = []
-            for _req_label in self.can_be_merged_required_labels:
-                if _req_label not in _labels:
-                    missing_required_labels.append(_req_label)
+            required_check_in_progress_failure_output, check_runs_in_progress = self._required_check_in_progress(
+                last_commit_check_runs=last_commit_check_runs
+            )
+            if required_check_in_progress_failure_output:
+                failure_output += required_check_in_progress_failure_output
 
-            if missing_required_labels:
-                failure_output += f"Missing required labels: {', '.join(missing_required_labels)}\n"
+            labels_failure_output = self._wip_or_hold_lables_exists(labels=_labels)
+            if labels_failure_output:
+                failure_output += labels_failure_output
 
-            pr_approved = False
-            for _label in _labels:
-                if APPROVED_BY_LABEL_PREFIX.lower() in _label.lower():
-                    approved_user = _label.split("-")[-1]
-                    if approved_user in self.approvers and self.parent_committer != approved_user:
-                        pr_approved = True
-                        break
+            required_check_failed_failure_output = self._required_check_failed(
+                last_commit_check_runs=last_commit_check_runs, check_runs_in_progress=check_runs_in_progress
+            )
+            if required_check_failed_failure_output:
+                failure_output += required_check_failed_failure_output
 
-            if not pr_approved:
-                missing_approvers = [approver for approver in self.approvers if approver != self.parent_committer]
-                failure_output += f"Missing lgtm/approved from approvers: {', '.join(missing_approvers)}\n"
+            lables_failue_output = self._check_lables_for_can_be_merged(labels=_labels)
+            if lables_failue_output:
+                failure_output += lables_failue_output
+
+            pr_approvered_failure_output = self._check_if_pr_approved(labels=_labels)
+            if pr_approvered_failure_output:
+                failure_output += pr_approvered_failure_output
 
             if not failure_output:
                 self._add_label(label=CAN_BE_MERGED_STR)
@@ -1497,7 +1416,7 @@ stderr: `{_err}`
             tag_name=tag,
             clone_repo_dir=clone_repo_dir,
         ):
-            build_rc, build_out, build_err = self.run_podman_command(command=podman_build_cmd)
+            build_rc, build_out, build_err = self.run_podman_command(command=podman_build_cmd, pipe=True)
             output: Dict[str, str] = {
                 "title": "Build container",
                 "summary": "",
@@ -1804,7 +1723,7 @@ stderr: `{_err}`
 
     def set_jira_in_pull_request(self) -> None:
         if self.jira_enabled_repository:
-            reviewers_and_approvers = self.reviewers + self.approvers
+            reviewers_and_approvers = self.root_reviewers + self.root_approvers
             if self.parent_committer in reviewers_and_approvers:
                 self.jira_assignee = self.jira_user_mapping.get(self.parent_committer)
                 if not self.jira_assignee:
@@ -2018,9 +1937,12 @@ Adding label/s `{" ".join([_cp_label for _cp_label in cp_labels])}` for automati
         try:
             self.logger.info(f"{self.log_prefix} Adding PR owner as assignee")
             self.pull_request.add_to_assignees()
-        except Exception:
-            if self.approvers:
-                self.pull_request.add_to_assignees(self.approvers[0])
+        except Exception as exp:
+            self.logger.debug(f"{self.log_prefix} Exception while adding PR owner as assignee: {exp}")
+
+            if self.root_approvers:
+                self.logger.debug(f"{self.log_prefix} Falling back to first approver as assignee")
+                self.pull_request.add_to_assignees(self.root_approvers[0])
 
     def set_pull_request_automerge(self) -> None:
         if self.parent_committer in self.auto_verified_and_merged_users:
@@ -2065,13 +1987,220 @@ Adding label/s `{" ".join([_cp_label for _cp_label in cp_labels])}` for automati
         shutil.rmtree("/tmp/storage-run-1000/containers", ignore_errors=True)
         shutil.rmtree("/tmp/storage-run-1000/libpod/tmp", ignore_errors=True)
 
-    def run_podman_command(self, command: str) -> Tuple[bool, str, str]:
-        rc, out, err = run_command(command=command, log_prefix=self.log_prefix)
+    def run_podman_command(self, command: str, pipe: bool = False) -> Tuple[bool, str, str]:
+        rc, out, err = run_command(command=command, log_prefix=self.log_prefix, pipe=pipe)
+
         if rc:
             return rc, out, err
 
         if self.is_podman_bug(err=err):
             self.fix_podman_bug()
-            return run_command(command=command, log_prefix=self.log_prefix)
+            return run_command(command=command, log_prefix=self.log_prefix, pipe=pipe)
 
         return rc, out, err
+
+    def get_all_approvers_and_reviewers(self) -> dict[str, dict[str, Any]]:
+        # Dictionary mapping OWNERS file paths to their approvers and reviewers
+        _owners: dict[str, dict[str, Any]] = {}
+
+        max_owners_files = 1000  # Configurable limit
+        owners_count = 0
+
+        tree = self.repository.get_git_tree(self.pull_request_branch, recursive=True)
+        for element in tree.tree:
+            if element.type == "blob" and element.path.endswith("OWNERS"):
+                owners_count += 1
+                if owners_count > max_owners_files:
+                    self.logger.error(f"{self.log_prefix} Too many OWNERS files (>{max_owners_files})")
+                    break
+
+                content_path = element.path
+                _path = self.repository.get_contents(content_path)
+                if isinstance(_path, list):
+                    _path = _path[0]
+
+                try:
+                    content = yaml.safe_load(_path.decoded_content)
+                    if self._validate_owners_content(content, content_path):
+                        parent_path = str(Path(content_path).parent)
+                        if not parent_path:
+                            parent_path = "."
+                        _owners[parent_path] = content
+
+                except yaml.YAMLError as exp:
+                    self.logger.error(f"{self.log_prefix} Invalid OWNERS file {content_path}: {exp}")
+                    continue
+
+        self.logger.debug(f"{self.log_prefix} Owners file mapping: {_owners}")
+        return _owners
+
+    def get_all_approvers(self) -> list[str]:
+        _approvers: list[str] = []
+        for list_of_approvers in self.owners_data_for_changed_files().values():
+            for _approver in list_of_approvers.get("approvers", []):
+                _approvers.append(_approver)
+
+        _approvers.sort()
+        return _approvers
+
+    def get_all_reviewers(self) -> list[str]:
+        _reviewers: list[str] = []
+        for list_of_reviewers in self.owners_data_for_changed_files().values():
+            for _approver in list_of_reviewers.get("reviewers", []):
+                _reviewers.append(_approver)
+
+        _reviewers.sort()
+        return _reviewers
+
+    def owners_data_for_changed_files(self) -> dict[str, dict[str, Any]]:
+        data: dict[str, dict[str, Any]] = {}
+
+        changed_folders = {Path(cf).parent for cf in self.changed_files}
+
+        changed_folder_match: list[str] = []
+
+        require_root_approvers: bool = False
+
+        for owners_dir, owners_data in self.all_approvers_and_reviewers.items():
+            _owners_dir = Path(owners_dir)
+
+            for changed_folder in changed_folders:
+                if _owners_dir == changed_folder or _owners_dir.parents == changed_folders:
+                    data[owners_dir] = owners_data
+                    changed_folder_match.append(owners_dir)
+                    if not require_root_approvers:
+                        require_root_approvers = owners_data.get("root-approvers", True)
+
+        if [_folder for _folder in changed_folders if str(_folder) not in changed_folder_match]:
+            data["."] = self.all_approvers_and_reviewers.get(".", {})
+
+        if require_root_approvers:
+            data["."] = self.all_approvers_and_reviewers.get(".", {})
+
+        return data
+
+    def _validate_owners_content(self, content: Any, path: str) -> bool:
+        """Validate OWNERS file content structure."""
+        try:
+            if not isinstance(content, dict):
+                raise ValueError("OWNERS file must contain a dictionary")
+
+            for key in ["approvers", "reviewers"]:
+                if key in content:
+                    if not isinstance(content[key], list):
+                        raise ValueError(f"{key} must be a list")
+
+                    if not all(isinstance(_elm, str) for _elm in content[key]):
+                        raise ValueError(f"All {key} must be strings")
+
+            return True
+
+        except ValueError as e:
+            self.logger.error(f"{self.log_prefix} Invalid OWNERS file {path}: {e}")
+            return False
+
+    def _required_check_in_progress(self, last_commit_check_runs: list[CheckRun]) -> tuple[str, list[str]]:
+        self.all_required_status_checks = self.get_all_required_status_checks()
+        last_commit_check_runs = list(self.last_commit.get_check_runs())
+        self.logger.debug(f"{self.log_prefix} Check if any required check runs in progress.")
+        check_runs_in_progress = [
+            check_run.name
+            for check_run in last_commit_check_runs
+            if check_run.status == IN_PROGRESS_STR
+            and check_run.name != CAN_BE_MERGED_STR
+            and check_run.name in self.all_required_status_checks
+        ]
+        if check_runs_in_progress:
+            self.logger.debug(
+                f"{self.log_prefix} Some required check runs in progress {check_runs_in_progress}, "
+                f"skipping check if {CAN_BE_MERGED_STR}."
+            )
+            return f"Some required check runs in progress {', '.join(check_runs_in_progress)}\n", check_runs_in_progress
+        return "", []
+
+    def _required_check_failed(self, last_commit_check_runs: list[CheckRun], check_runs_in_progress: list[str]) -> str:
+        failed_check_runs = []
+        for check_run in last_commit_check_runs:
+            if (
+                check_run.name == CAN_BE_MERGED_STR
+                or check_run.conclusion == SUCCESS_STR
+                or check_run.conclusion == QUEUED_STR
+                or check_run.name not in self.all_required_status_checks
+            ):
+                continue
+
+            failed_check_runs.append(check_run.name)
+
+        if failed_check_runs:
+            exclude_in_progress = [
+                failed_check_run
+                for failed_check_run in failed_check_runs
+                if failed_check_run not in check_runs_in_progress
+            ]
+            return f"Some check runs failed: {', '.join(exclude_in_progress)}\n"
+
+        return ""
+
+    def _wip_or_hold_lables_exists(self, labels: list[str]) -> str:
+        failure_output = ""
+        is_hold = HOLD_LABEL_STR in labels
+        is_wip = WIP_STR in labels
+
+        if is_hold or is_wip:
+            if is_hold:
+                failure_output += "Hold label exists.\n"
+
+            if is_wip:
+                failure_output += "WIP label exists.\n"
+
+        return failure_output
+
+    def _check_lables_for_can_be_merged(self, labels: list[str]) -> str:
+        failure_output = ""
+
+        for _label in labels:
+            if CHANGED_REQUESTED_BY_LABEL_PREFIX.lower() in _label.lower():
+                change_request_user = _label.split("-")[-1]
+                if change_request_user in self.all_approvers:
+                    failure_output += "PR has changed requests from approvers\n"
+
+        missing_required_labels = []
+        for _req_label in self.can_be_merged_required_labels:
+            if _req_label not in labels:
+                missing_required_labels.append(_req_label)
+
+        if missing_required_labels:
+            failure_output += f"Missing required labels: {', '.join(missing_required_labels)}\n"
+
+        return failure_output
+
+    def _check_if_pr_approved(self, labels: list[str]) -> str:
+        approved_by = []
+        for _label in labels:
+            if APPROVED_BY_LABEL_PREFIX.lower() in _label.lower():
+                approved_user = _label.split("-")[-1]
+                if self.parent_committer == approved_user:
+                    continue
+
+                approved_by.append(approved_user)
+
+        missing_approvers = self.all_approvers.copy()
+
+        for data in self.owners_data_for_changed_files().values():
+            required_pr_approvers = data.get("approvers", [])
+            for required_pr_approver in required_pr_approvers:
+                if required_pr_approver in approved_by:
+                    # Once we found approver in approved_by list, we remove all approvers from missing_approvers list for this owners file
+                    {
+                        missing_approvers.remove(_approver)  # type: ignore
+                        for _approver in required_pr_approvers
+                        if _approver in missing_approvers
+                    }
+                    break
+
+        missing_approvers = list(set(missing_approvers))
+
+        if missing_approvers:
+            return f"Missing lgtm/approved from approvers: {', '.join(missing_approvers)}\n"
+
+        return ""
